//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Functions that process commands sent from the server.


// This function is for chat messages only; it is invoked on the client when
// the server does a commandToClient with the tag ChatMessage.  (Cf. the
// functions chatMessage* in common/server/message.cs.)

// This just invokes onChatMessage, which the mod code must define.

function clientCmdChatMessage(%sender, %voice, %pitch, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10)
{
   onChatMessage(detag(%msgString), %voice, %pitch);
}

// Game event descriptions, which may or may not include text messages, can be
// sent using the message* functions in common/server/message.cs.  Those
// functions do commandToClient with the tag ServerMessage, which invokes the
// function below.

// For ServerMessage messages, the client can install callbacks that will be
// run, according to the "type" of the message.

function clientCmdServerMessage(%msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10)
{
   // Get the message type; terminates at any whitespace.
   %tag = getWord(%msgType, 0);

   // First see if there is a callback installed that doesn't have a type;
   // if so, that callback is always executed when a message arrives.
   for (%i = 0; (%func = $MSGCB["", %i]) !$= ""; %i++) {
      call(%func, %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10);
   }

   // Next look for a callback for this particular type of ServerMessage.
   if (%tag !$= "") {
      for (%i = 0; (%func = $MSGCB[%tag, %i]) !$= ""; %i++) {
         call(%func, %msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10);
      }
   }
}

// Called by the client to install a callback for a particular type of
// ServerMessage.
function addMessageCallback(%msgType, %func)
{
   for (%i = 0; (%afunc = $MSGCB[%msgType, %i]) !$= ""; %i++) {
      // If it already exists as a callback for this type,
      // nothing to do.
      if (%afunc $= %func) {
         return;
      }
   }
   // Set it up.
   $MSGCB[%msgType, %i] = %func;
}



// The following is the callback that will be executed for every ServerMessage,
// because we're going to install it without a specified type.  Any type-
// specific callbacks will be executed afterward.

// This just invokes onServerMessage, which the mod code must define.

function defaultMessageCallback(%msgType, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10)
{
   onServerMessage(detag(%msgString));
}

// Register that default message handler now.
addMessageCallback("", defaultMessageCallback);

//----------------------------------------------------------------------------------------------
// COGMAC
//----------------------------------------------------------------------------------------------

function clientCmdChatMessageOther(%sender, %voice, %pitch, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10)
{
   %resolution = PlayGui.getExtent();
   %resX = getWord(%resolution, 0);
   %resY = getWord(%resolution, 1);

   if( !bmpChatOtherText.isVisible() )
   {
      bmpChatOtherText.setPosition( 0, 0 );
      bmpChatOtherText.setExtent( %resX, 66 );
      bmpChatOtherText.setVisible( true );

      txtChatOther.setVisible( true );
      txtChatOtherSmall.setVisible( false );
      txtChatOtherSmall2.setVisible( false );
      txtChatOtherSmall3.setVisible( false );

      txtChatOther.setPosition( 0, -8 );
      txtChatOtherSmall.setPosition( 0, -8 );
      txtChatOtherSmall2.setPosition( 0, 0 );
      txtChatOtherSmall3.setPosition( 0, 0 );
   }

   %text = stripColorCodes( detag(%msgString) );
   txtChatOther.setText( %text );
   txtChatOtherSmall.setText( %text );

   // Go smaller.
   if( txtChatOther.getTextWidth() > %resX )
   {
      txtChatOther.setVisible( false );
      txtChatOtherSmall.setVisible( true );
   }

   // Go even smaller.
   if( txtChatOtherSmall.getTextWidth() > %resX )
   {
      txtChatOtherSmall.setVisible( false );
      txtChatOtherSmall2.setVisible( true );
      txtChatOtherSmall2.setText( %text );
   }

   // Go even smaller.
   if( txtChatOtherSmall2.getTextWidth() > %resX )
   {
      txtChatOtherSmall2.setVisible( false );
      txtChatOtherSmall3.setVisible( true );
      txtChatOtherSmall3.setText( %text );
   }

   txtChatOther.expire = txtChatOther.expire + 1;
   schedule(6000, 0, ExpireChatOther, txtChatOther.expire );
}

function ExpireChatOther( %i )
{
   if( txtChatOther.expire != %i ) return;
   bmpChatOtherText.setVisible( false );
   txtChatOther.setVisible( false );
   txtChatOtherSmall.setVisible( false );
}

function clientCmdSetNarration(%msg)
{
   if( !$GameplayEnabled ) return;

   txtNarration.setPosition( 0, 66 );
   txtNarration.setText( %msg );
   txtNarration.setVisible(true);
   
   txtNarration.expire = txtNarration.expire + 1;
   schedule( 5000, 0, ExpireNarration, txtNarration.expire );
}

function ExpireNarration( %i )
{
   if( txtNarration.expire != %i ) return;
   txtNarration.setVisible(false);
}

function clientCmdToggleTypingIndicator(%sender, %val)
{
   if( %val == true )
   {
      txtNarration.setPosition( 0, 66 );
      txtNarration.setText( "[" @ %sender SPC "is typing...]" );
      txtNarration.setVisible(true);
      txtNarration.expire = 0;
   }
   else txtNarration.setVisible(false);
}

function clientCmdToggleNoListenerWarning(%val)
{
   if( %val == false )
   {
      %resolution = PlayGui.getExtent();
      %resX = getWord(%resolution, 0);
      %resY = getWord(%resolution, 1);

      txtNoListenerWarning.setPosition( 0, %resY - 66 - 40 );
      txtNoListenerWarning.setText( "[Face the character that you would like to speak to]" );
      txtNoListenerWarning.setVisible(true);
   }
   else txtNoListenerWarning.setVisible(false);
}

//----------------------------------------------------------------------------------------------
// COGMAC
//----------------------------------------------------------------------------------------------















