//-------------------------------------------------------------------------------
// DBRegister
//-------------------------------------------------------------------------------

datablock StaticShapeData(DBRegister : DBObject)
{
   // The category variable determines where the item
   // shows up in the mission editor's creator tree.
   // Path is relative to where the script is executed from.
   category = "Restaurant";
   shapeFile = "~/data/shapes/restaurant/cash_register/cash_register.dts";
   bitmap = "restaurant/data/shapes/restaurant/cash_register/cash_register.bmp";
   noun = "cash register";
};

function DBRegister::onAdd(%this, %obj) { DBObject::OnAdd(%this, %obj); }
function DBRegister::OnPickup( %this, %client, %user, %obj ) { DBObject::OnPickup( %this, %client, %user, %obj ); }
function DBRegister::OnPutDown( %this, %client, %user, %obj, %surface, %pos, %ignore ) { DBObject::OnPutDown( %this, %client, %user, %obj, %surface, %pos, %ignore ); }
function DBRegister::OnLookAt( %this, %client, %obj ) { DBObject::OnLookAt( %this, %client, %obj ); }
function DBRegister::OnGive( %this, %client, %user, %obj, %target, %pos ) { DBObject::OnGive( %this, %client, %user, %obj, %target, %pos ); }
function DBRegister::OnSit( %this, %client, %user, %obj ) { DBObject::OnSit( %this, %client, %user, %obj ); }
function DBRegister::OnEat( %this, %client, %obj ) { DBObject::OnEat( %this, %client, %obj ); }

function DBRegister::OnTouch( %this, %client, %user, %obj )
{
   COGLOG_STATECHANGE( %client.player, "ATPOS", COGLOG_POS(%client.player.getPosition()) );
   COGLOG_ACTION( %client.player, "TOUCH", %obj );

   CommandToClient( %client, 'ShowRegisterGui' );

   // Play a touch sound.
   %soundTouch = "~/data/sound/cogmac/print_receipt.wav";
   PlaySound( %soundTouch, %user.getPosition() );
}

//-------------------------------------------------------------------------------
// DBBill
//-------------------------------------------------------------------------------

datablock StaticShapeData(DBBill : DBObject)
{
   // The category variable determines where the item
   // shows up in the mission editor's creator tree.
   // Path is relative to where the script is executed from.
   category = "Restaurant";
   shapeFile = "~/data/shapes/restaurant/bill/bill.dts";
   bitmap = "";
   soundPutDown = "~/data/sound/cogmac/pages.wav";
   noun = "bill";
};

function DBBill::onAdd(%this, %obj) { DBObject::OnAdd(%this, %obj); }
function DBBill::OnPickup( %this, %client, %user, %obj ) { DBObject::OnPickup( %this, %client, %user, %obj ); }
function DBBill::OnGive( %this, %client, %user, %obj, %target, %pos ) { DBObject::OnGive( %this, %client, %user, %obj, %target, %pos ); }
function DBBill::OnSit( %this, %client, %user, %obj ) { DBObject::OnSit( %this, %client, %user, %obj ); }
function DBBill::OnEat( %this, %client, %obj ) { DBObject::OnEat( %this, %client, %obj ); }

function DBBill::OnLookAt( %this, %client, %obj )
{
   COGLOG_STATECHANGE( %client.player, "ATPOS", COGLOG_POS(%client.player.getPosition()) );
   COGLOG_ACTION( %client.player, "LOOKAT", %obj );
   CommandToClient( %client, 'ShowBillGui', %obj,
                    %obj.lstItems[0], %obj.lstItems[1], %obj.lstItems[2], %obj.lstItems[3], %obj.lstItems[4], 
                    %obj.lstItems[5], %obj.lstItems[6], %obj.lstItems[7], %obj.lstItems[8], %obj.lstItems[9], 
                    %obj.subtotal, %obj.tax, %obj.tip, %obj.total, %obj.paid );
}

function DBBill::OnPutDown( %this, %client, %user, %obj, %surface, %pos, %ignore )
{
   COGLOG_STATECHANGE( %client.player, "ATPOS", COGLOG_POS(%client.player.getPosition()) );

   // Delete bills when returned to the register.
   if( %obj.paid && ( %surface.getDatablock().getName() $= "DBRegister" ) )
   {
      // Confirm deposit if payer and payee are the same player.
      if( %obj.payer $= %client.player.getName() )
      {
         CommandToClient( %client, 'ConfirmDeposit', %pos );
         return;
      }

      // Player earns money.
      CompleteDeposit( %client );
      return;
   }

   DBObject::OnPutDown( %this, %client, %user, %obj, %surface, %pos, %ignore ); 
}

function serverCmdCompleteDeposit( %client )
{
   CompleteDeposit( %client );
}

function CompleteDeposit( %client )
{
   // Play a putdown sound.
   PlaySound( "~/data/sound/cogmac/print_receipt.wav", %client.player.getPosition() );

   // Set the payee to whoever is depositing the check.
   %surface = %client.getSelectedObject();
   %obj = %client.player.getMountedObject( 0 );
   %obj.payee = %client.player.getName();

   schedule( 100, 0, NotifyEarnMoney, %client, %obj );

   COGLOG_ACTION( %client.player, "PUTDOWN", %obj );
   COGLOG_STATECHANGE( %obj, "ATTACHEDTO", "NULL" );
   COGLOG_STATECHANGE( %obj, "ON", COGLOG_ID(%surface) );

   CommandToClient( %client, 'CleanupPopup' );
}

function NotifyEarnMoney( %client, %obj )
{
   %count = ClientGroup.getCount();
   for ( %i = 0; %i < %count; %i++ )
   {
      // Find the payee's client.
      %objClient = ClientGroup.getObject( %i );
      if( %objClient.player.getName() $= %obj.payee )
      {
         %payeeClient = %objClient;
         break;
      }
   }

   // Payee earns money.
   %total = %obj.actualTip + (getSubStr(%obj.actualSubtotal, 1, 255) * 0.50);
   SetCurrentBalance(%payeeClient, %payeeClient.money + %total);
   SetNarration( %payeeClient, "[" @ %payeeClient.player.getName() SPC "earned" SPC FormatCashOutput(%total) SPC "(50% of" SPC FormatCashOutput(getSubStr(%obj.actualSubtotal, 1, 255)) SPC "plus" SPC FormatCashOutput(%obj.actualTip) SPC "tip)]" );

   CommandToClient( %payeeClient, 'ShowEarnMoneyGui', %obj.payer, %obj.payee,
 				    %obj.actualSubtotal, %obj.actualTip, %payeeClient.money, %obj.getPosition() );

   COGLOG_EVENT( %payeeClient.player, "EARNED", FormatCashOutput(%total) );
   COGLOG_STATECHANGE( %obj, "DELETED", "TRUE" );
   %obj.delete();
}

function DBBill::OnTouch( %this, %client, %user, %obj ) { DBBill::OnLookAt( %this, %client, %obj ); }

//-------------------------------------------------------------------------------

function ServerCmdProcessBillPayment( %client, %bill, %tip, %total ) 
{
   // Strip the '$' from the total.
   %total = getSubStr( %total, 1, 99999 );
   %subtotal = getSubStr( %bill.subtotal, 1, 99999 );
   %bal = %client.money - %total;

   %totalPaid = %total;
   %actualTip = %tip;
   %actualSubtotal = %subtotal;

   // Payer does not have enough money to pay.
   if( %bal < 0 )
   {
      COGLOG_ACTION( %client.player, "FAILPAY", %bill );
      CommandToClient( %client, 'WarnCantPay', %bill.getPosition(), %client.money, %total );

      SetNarration( %client, "[" @ %client.player.getName() SPC "paid" SPC FormatCashOutput(%client.money) SPC "of the" SPC FormatCashOutput(%total) SPC "bill, and ran out of money!]" );
      %totalPaid = %client.money;
      %bal = 0;

      // Calculate the tip actually paid.
      %actualTip = (%client.money - %total) + %tip;
      if( %actualTip < 0 ) %actualTip = 0;

      // Calculate the subtotal actually paid.
      if( (%actualTip == 0) && ( %subtotal > %client.money ) )
      {
         %actualSubtotal = %client.money;
      }
   }

   // Pay the bill!
   %bill.tip = %tip;
   %bill.total = "$" @ %total;
   %bill.paid = true;
   %bill.payer = %client.player.getName();
   %bill.actualTip = %actualTip;
   %bill.actualSubtotal = "$" @ %actualSubtotal;

   COGLOG_ACTION( %client.player, "PAY", %bill );
   COGLOG_STATECHANGE( %bill, "TIP_DUE", %tip );
   COGLOG_STATECHANGE( %bill, "TIP_PAID", %actualTip );
   COGLOG_STATECHANGE( %bill, "SUBTOTAL_DUE", %subtotal );
   COGLOG_STATECHANGE( %bill, "SUBTOTAL_PAID", %actualSubtotal );
   COGLOG_STATECHANGE( %bill, "TOTAL_DUE", %total );
   COGLOG_STATECHANGE( %bill, "TOTAL_PAID", %totalPaid );
   COGLOG_STATECHANGE( %bill, "PAID", "TRUE" );

   // Deduct balance from payer's account.
   SetCurrentBalance(%client, %bal);

   // Play a pay sound.
   %soundPay = "~/data/sound/cogmac/cashreg.wav";
   PlaySound( %soundPay, %bill.getPosition() );

   if( %total == %totalPaid ) CommandToClient( %client, 'CleanupPopup' );
   else CommandToClient( %client, 'ClosePaymentDlg' );
}

//-------------------------------------------------------------------------------

function ServerCmdSpawnNewBill( %client, %tax, %subtotal, %total,
                                %lstItems0, %lstItems1, %lstItems2, %lstItems3, %lstItems4, 
                                %lstItems5, %lstItems6, %lstItems7, %lstItems8, %lstItems9 )
{
   // Spawn a bill.
   %bill = new StaticShape() {
      dataBlock = DBBill;
   };

   // Place bill in the world.
   %groupName = "MissionGroup/BillSpawnPoints";
   %group = nameToID(%groupName);
   if (%group != -1) {
         %spawn = %group.getObject( 0 );
		 %bill.setTransform( %spawn.getTransform() );

         %searchMasks = $TypeMasks::ItemObjectType | $TypeMasks::StaticTSObjectType | 
                        $TypeMasks::StaticShapeObjectType | $TypeMasks::TerrainObjectType | 
                        $TypeMasks::InteriorObjectType;

         // Pick a random drop position.
         %billTrigger = nameToId("billTrigger");
         %pos = %billTrigger.getRandomContainedPos();
         %end = VectorAdd(%pos, "0 0 -100");
         %scanTarg = ContainerRayCast (%pos, %end, %searchMasks);
         %drop = getWords( %scanTarg, 1, 3 );
         %bill.setPosition( %drop );

         // Play a spawn sound.
         %soundSpawn = "~/data/sound/cogmac/magic_bong.wav";
         PlaySound( %soundSpawn, %drop );

         COGLOG_STATECHANGE( %bill, "ATPOS", COGLOG_POS(%drop) );
         COGLOG_STATECHANGE( %bill, "ON", COGLOG_ID(getWords( %scanTarg, 0)) );
   }

   // Set the bill's data.
   %bill.lstItems[0] = %lstItems0;
   %bill.lstItems[1] = %lstItems1;
   %bill.lstItems[2] = %lstItems2;
   %bill.lstItems[3] = %lstItems3;
   %bill.lstItems[4] = %lstItems4;
   %bill.lstItems[5] = %lstItems5;
   %bill.lstItems[6] = %lstItems6;
   %bill.lstItems[7] = %lstItems7;
   %bill.lstItems[8] = %lstItems8;
   %bill.lstItems[9] = %lstItems9;

   COGLOG_STATECHANGE( %bill, "ITEM0", %lstItems0 );
   COGLOG_STATECHANGE( %bill, "ITEM1", %lstItems1 );
   COGLOG_STATECHANGE( %bill, "ITEM2", %lstItems2 );
   COGLOG_STATECHANGE( %bill, "ITEM3", %lstItems3 );
   COGLOG_STATECHANGE( %bill, "ITEM4", %lstItems4 );
   COGLOG_STATECHANGE( %bill, "ITEM5", %lstItems5 );
   COGLOG_STATECHANGE( %bill, "ITEM6", %lstItems6 );
   COGLOG_STATECHANGE( %bill, "ITEM7", %lstItems7 );
   COGLOG_STATECHANGE( %bill, "ITEM8", %lstItems8 );
   COGLOG_STATECHANGE( %bill, "ITEM9", %lstItems9 );

   if( %bill.lstItems[0] !$= "" ) %bill.lstCosts[0] = GetItemPrice( $menuItemList, %bill.lstItems[0] );
   if( %bill.lstItems[1] !$= "" ) %bill.lstCosts[1] = GetItemPrice( $menuItemList, %bill.lstItems[1] );
   if( %bill.lstItems[2] !$= "" ) %bill.lstCosts[2] = GetItemPrice( $menuItemList, %bill.lstItems[2] );
   if( %bill.lstItems[3] !$= "" ) %bill.lstCosts[3] = GetItemPrice( $menuItemList, %bill.lstItems[3] );
   if( %bill.lstItems[4] !$= "" ) %bill.lstCosts[4] = GetItemPrice( $menuItemList, %bill.lstItems[4] );
   if( %bill.lstItems[5] !$= "" ) %bill.lstCosts[5] = GetItemPrice( $menuItemList, %bill.lstItems[5] );
   if( %bill.lstItems[6] !$= "" ) %bill.lstCosts[6] = GetItemPrice( $menuItemList, %bill.lstItems[6] );
   if( %bill.lstItems[7] !$= "" ) %bill.lstCosts[7] = GetItemPrice( $menuItemList, %bill.lstItems[7] );
   if( %bill.lstItems[8] !$= "" ) %bill.lstCosts[8] = GetItemPrice( $menuItemList, %bill.lstItems[8] );
   if( %bill.lstItems[9] !$= "" ) %bill.lstCosts[9] = GetItemPrice( $menuItemList, %bill.lstItems[9] );

   %bill.tax = %tax;
   %bill.subtotal = %subtotal;
   %bill.tip = "0.00";
   %bill.total = %total;
   %bill.paid = false;
   %bill.payee = %client.player.getName();

   COGLOG_STATECHANGE( %bill, "TAX", %tax );
   COGLOG_STATECHANGE( %bill, "SUBTOTAL", %subtotal );
   COGLOG_STATECHANGE( %bill, "TIP", "0.00" );
   COGLOG_STATECHANGE( %bill, "TOTAL", %total );
   COGLOG_STATECHANGE( %bill, "PAID", "FALSE" );
}













